#ifndef OS_H_
#define OS_H_

// ARM definitions
#define  ARM_MODE_ARM           0x00000000
#define  ARM_SVC_MODE_ARM      (0x00000013L + ARM_MODE_ARM)

// Program specific
#define STACKSIZE 256
#define NUMBER_OF_TASKS 3

// Critical Section Functions
#define CPU_SR int

CPU_SR  CPU_SR_Save   (void);
void    CPU_SR_Restore(CPU_SR cpu_sr);

#define ENTER_CRITICAL()  {cpu_sr = CPU_SR_Save();}
#define EXIT_CRITICAL()   {CPU_SR_Restore(cpu_sr);}

// Task context management
//
// To manage the context for the tasks all we really need
// is to store the stack pointers. The actual context for the
// tasks are stored on their own stacks.
//
// Task Control Block. Use to save information about a task.
typedef struct
{
    int stackPointer;   // Task's stack pointer. Where the context is saved.
    int delayCount;     // Delay count. Task is delayed if count > 0.
} TaskBlock;

void initialize_timer_tick (void);   // Intializes timer for the interrupt tick

void switch_to_current(void);        // Switches to the current task
void SwitchContext(void);            // Switches context by saving old and restoring new

void CreateTask(int id, int* stack, void* task_address);
void StartOS(void);
void Scheduler(int sp);

// Delay a task by number of ticks
void DelayTask(int taskId, int numOfTicks);

#endif /* OS_H_ */
